import { ReadableStream } from 'ts-stream';
import { SnowflakeObject, Constructor } from './mapping/mapper';
import { SqlCommand, ConnectionOptions, SnowflakeConnection, GlobalConfigOptions } from './snowflake-core';
import { Query } from './queries';
import './stream-extensions';
import './promise-extensions';
export interface Logger {
    info(msg: string): void;
    error(msg: string): void;
    log(msg: string): void;
    warn(msg: string): void;
}
export declare class SnowflakeClient {
    private readonly conn;
    private readonly logger;
    static configure(opts: Partial<Required<GlobalConfigOptions>>): void;
    constructor(connection: SnowflakeConnection, logger: Logger);
    static createConnection(opts: ConnectionOptions, logger?: Logger): Promise<SnowflakeClient>;
    execute(args: SqlCommand | string): Promise<void>;
    write<T>(table: Constructor<T>, items: Array<SnowflakeObject<T>>): Promise<void>;
    writeStream<T>(table: Constructor<T>, stream: ReadableStream<SnowflakeObject<T>>, batchSize: number): Promise<void>;
    sink<T>(table: Constructor<T>, batchSize: number): (stream: ReadableStream<SnowflakeObject<T>>) => Promise<void>;
    readStreamOf<T>(table: Constructor<T>, query?: Query<T>): ReadableStream<T>;
    readStream<T>(table: Constructor<T>, sql: SqlCommand | string): ReadableStream<T>;
    readStreamRaw(sql: SqlCommand | string): ReadableStream<any>;
}
